//
// by. S. Gehlbach
// console routines
// edit with :set ts=4
//
#define asm __asm__
#define volatile __volatile__
#define inline __inline__

/* Base Address */
#ifndef TTYS0_BASE
#define TTYS0_BASE 0x3f8
#endif

#ifndef TTYS0_BAUD
#define TTYS0_BAUD 115200
#endif

#if ((115200%TTYS0_BAUD) != 0)
#error Bad ttys0 baud rate
#endif

#define TTYS0_DIV	(115200/TTYS0_BAUD)

/* Line Control Settings */
#ifndef TTYS0_LCS
/* Set 8bit, 1 stop bit, no parity */
#define TTYS0_LCS	0x3
#endif

#define UART_LCS	TTYS0_LCS

/* Data */
#define UART_RBR 0x00
#define UART_TBR 0x00

/* Control */
#define UART_IER 0x01
#define UART_IIR 0x02
#define UART_FCR 0x02
#define UART_LCR 0x03
#define UART_MCR 0x04
#define UART_DLL 0x00
#define UART_DLM 0x01

/* Status */
#define UART_LSR 0x05
#define UART_MSR 0x06
#define UART_SCR 0x07

#ifndef COMMON
#include "common.h"
#endif

extern inline int uart_can_tx_byte()
{
	return inb(TTYS0_BASE + UART_LSR) & 0x20;
}

extern inline void uart_wait_to_tx_byte()
{
	while(!uart_can_tx_byte())
		;
}

extern inline void uart_wait_until_sent()
{
	while(!(inb(TTYS0_BASE + UART_LSR) & 0x40)) 
		;
}

extern inline void uart_tx_byte(unsigned char data)
{
	uart_wait_to_tx_byte();
	outb(data, TTYS0_BASE + UART_TBR);
	/* Make certain the data clears the fifos */
	uart_wait_until_sent();
}

extern inline int uart_have_rx_byte()
{
	return inb(TTYS0_BASE + UART_LSR) & 0x1;
}

extern inline void uart_enable_rx_byte()
{
	/* say we are ready for a byte */
	outb(0x02 | inb(TTYS0_BASE + UART_MCR), TTYS0_BASE + UART_MCR);
}

extern inline void uart_disable_rx_byte()
{
	/* say we aren't ready for another byte */
	outb(~0x02 & inb(TTYS0_BASE + UART_MCR), TTYS0_BASE + UART_MCR);
}

extern inline void uart_wait_for_rx_byte()
{
	uart_enable_rx_byte(TTYS0_BASE);
	while(!uart_have_rx_byte(TTYS0_BASE))
		;
	uart_disable_rx_byte();
}

extern inline unsigned char uart_rx_byte()
{
	static unsigned char data asm ("cl");
	if (!uart_have_rx_byte(TTYS0_BASE)) {
		uart_wait_for_rx_byte(TTYS0_BASE);
	}
	data = inb(TTYS0_BASE + UART_RBR);
	return data;
}

extern inline void uart_init()
{
	/* disable interrupts */
	outb(0x0, TTYS0_BASE + UART_IER);
	/* enable fifo's */
	outb(0x01, TTYS0_BASE + UART_FCR);
	/* Set Baud Rate Divisor to 12 ==> 115200 Baud */
	outb(0x80 | UART_LCS, TTYS0_BASE + UART_LCR);
	outb(TTYS0_DIV & 0xFF,   TTYS0_BASE + UART_DLL);
	outb((TTYS0_DIV >> 8) & 0xFF,    TTYS0_BASE + UART_DLM);
	outb(UART_LCS, TTYS0_BASE + UART_LCR);
}

extern inline void uart_tx_bytes(char *data, unsigned len)
{
	do {
		uart_wait_to_tx_byte();
		outb(*data, TTYS0_BASE + UART_TBR);
		data++;
		len--;
	} while(len);
	uart_wait_until_sent();
}

extern inline unsigned long uart_rx_bytes(char * buffer, unsigned long size)
{
	unsigned long bytes asm ("ecx");
	bytes = 0;
	if (size == 0) {
		return 0;
	}
	if (!uart_have_rx_byte()) {
		uart_wait_for_rx_byte();
	}
	do {
		buffer[bytes++] = inb(TTYS0_BASE + UART_RBR);
	} while((bytes < size) && uart_have_rx_byte());
	return bytes;
}
